@@ -3,7 +3,7 @@
 //
 // MIT License
 //
-// Copyright (c) 2011-2018 Michael Truog <mjtruog at protonmail dot com>
+// Copyright (c) 2011-2019 Michael Truog <mjtruog at protonmail dot com>
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -1475,7 +1475,15 @@ public class API
         }
     }
 
-    public Object poll()
+    /**
+     * Blocks to process incoming CloudI service requests
+     *
+     * @return false
+     * @throws InvalidInputException invalid input to internal function call
+     * @throws MessageDecodingException service messaging failure
+     * @throws TerminateException service execution must terminate
+     */
+    public boolean poll()
         throws InvalidInputException,
                MessageDecodingException,
                TerminateException
@@ -1483,12 +1491,46 @@ public class API
         return poll(-1);
     }
 
-    public Object poll(final int timeout)
+    /**
+     * Blocks to process incoming CloudI service requests
+     *
+     * @param timeout  the block timeout in milliseconds (-1 is infinity)
+     * @return a boolean to determine if a timeout occurred
+     * @throws InvalidInputException invalid input to internal function call
+     * @throws MessageDecodingException service messaging failure
+     * @throws TerminateException service execution must terminate
+     */
+    public boolean poll(final int timeout)
         throws InvalidInputException,
                MessageDecodingException,
                TerminateException
     {
-        return poll_request(timeout, true);
+        if (Boolean.TRUE == poll_request(timeout, true))
+            return true;
+        return false;
+    }
+
+    /**
+     * Shutdown the service successfully
+     */
+    public void shutdown()
+    {
+        shutdown("");
+    }
+
+    /**
+     * Shutdown the service successfully
+     *
+     * @param  reason      the shutdown reason
+     */
+    public void shutdown(final String reason)
+    {
+        OtpOutputStream shutdown = new OtpOutputStream();
+        shutdown.write(OtpExternal.versionTag);
+        final OtpErlangObject[] tuple = {new OtpErlangAtom("shutdown"),
+                                         new OtpErlangString(reason)};
+        shutdown.write_any(new OtpErlangTuple(tuple));
+        send(shutdown);
     }
 
     private HashMap<String,
