@@ -3,41 +3,27 @@
 
 (*
 
-  BSD LICENSE
+  MIT License
 
-  Copyright (c) 2017, Michael Truog <mjtruog at gmail dot com>
-  All rights reserved.
+  Copyright (c) 2017-2019 Michael Truog <mjtruog at protonmail dot com>
 
-  Redistribution and use in source and binary forms, with or without
-  modification, are permitted provided that the following conditions are met:
+  Permission is hereby granted, free of charge, to any person obtaining a
+  copy of this software and associated documentation files (the "Software"),
+  to deal in the Software without restriction, including without limitation
+  the rights to use, copy, modify, merge, publish, distribute, sublicense,
+  and/or sell copies of the Software, and to permit persons to whom the
+  Software is furnished to do so, subject to the following conditions:
 
-      * Redistributions of source code must retain the above copyright
-        notice, this list of conditions and the following disclaimer.
-      * Redistributions in binary form must reproduce the above copyright
-        notice, this list of conditions and the following disclaimer in
-        the documentation and/or other materials provided with the
-        distribution.
-      * All advertising materials mentioning features or use of this
-        software must display the following acknowledgment:
-          This product includes software developed by Michael Truog
-      * The name of the author may not be used to endorse or promote
-        products derived from this software without specific prior
-        written permission
+  The above copyright notice and this permission notice shall be included in
+  all copies or substantial portions of the Software.
 
-  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
-  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
-  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
-  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
-  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
-  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
-  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
-  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
-  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
-  DAMAGE.
+  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+  DEALINGS IN THE SOFTWARE.
 
  *)
 
@@ -98,9 +84,6 @@ module Instance = struct
       mutable timeout_sync : int;
       mutable timeout_terminate : int;
       mutable priority_default : int;
-      mutable request_timeout_adjustment : bool;
-      mutable request_timer : float;
-      mutable request_timeout : int;
       mutable response_info : string;
       mutable response : string;
       mutable trans_id : string;
@@ -123,9 +106,6 @@ module Instance = struct
      timeout_sync = 0;
      timeout_terminate;
      priority_default = 0;
-     request_timeout_adjustment = false;
-     request_timer = 0.0;
-     request_timeout = 0;
      response_info = "";
      response = "";
      trans_id = "";
@@ -134,7 +114,7 @@ module Instance = struct
   let init api process_index process_count
     process_count_max process_count_min prefix
     timeout_initialize timeout_async timeout_sync timeout_terminate
-    priority_default request_timeout_adjustment =
+    priority_default =
     api.process_index <- process_index ;
     api.process_count <- process_count ;
     api.process_count_max <- process_count_max ;
@@ -145,15 +125,13 @@ module Instance = struct
     api.timeout_sync <- timeout_sync ;
     api.timeout_terminate <- timeout_terminate ;
     api.priority_default <- priority_default ;
-    api.request_timeout_adjustment <- request_timeout_adjustment ;
     ()
   let reinit api process_count timeout_async timeout_sync
-    priority_default request_timeout_adjustment =
+    priority_default =
     api.process_count <- process_count ;
     api.timeout_async <- timeout_async ;
     api.timeout_sync <- timeout_sync ;
     api.priority_default <- priority_default ;
-    api.request_timeout_adjustment <- request_timeout_adjustment ;
     ()
   let set_response api response_info response trans_id =
     api.response_info <- response_info ;
@@ -184,7 +162,7 @@ module Instance = struct
   let callbacks_remove api pattern =
     let key = api.prefix ^ pattern in
     let value = Hashtbl.find api.callbacks key in
-    let _ = Queue.pop value in
+    let _f = Queue.pop value in
     if Queue.is_empty value then
       Hashtbl.remove api.callbacks key ;
     ()
@@ -385,28 +363,13 @@ let recv api : (string * int, string) result =
 let forward_async_i
   api name request_info request timeout priority trans_id pid :
   (unit, string) result =
-  let {Instance.request_timeout_adjustment;
-    request_timer; request_timeout; _} = api in
-  let timeout_new =
-    if request_timeout_adjustment && timeout = request_timeout then
-      let elapsed = truncate
-        (((Unix.gettimeofday ()) -. request_timer) *. 1000.0) in
-      if elapsed < 0 then
-        timeout
-      else if elapsed >= timeout then
-        0
-      else
-        timeout - elapsed
-    else
-      timeout
-  in
   match Erlang.term_to_binary (
     Erlang.OtpErlangTuple ([
       Erlang.OtpErlangAtom ("forward_async");
       Erlang.OtpErlangString (name);
       Erlang.OtpErlangBinary (request_info);
       Erlang.OtpErlangBinary (request);
-      Erlang.OtpErlangInteger (timeout_new);
+      Erlang.OtpErlangInteger (timeout);
       Erlang.OtpErlangInteger (priority);
       Erlang.OtpErlangBinary (trans_id);
       Erlang.OtpErlangPid (pid)])) with
@@ -428,28 +391,13 @@ let forward_async
 let forward_sync_i
   api name request_info request timeout priority trans_id pid :
   (unit, string) result =
-  let {Instance.request_timeout_adjustment;
-    request_timer; request_timeout; _} = api in
-  let timeout_new =
-    if request_timeout_adjustment && timeout = request_timeout then
-      let elapsed = truncate
-        (((Unix.gettimeofday ()) -. request_timer) *. 1000.0) in
-      if elapsed < 0 then
-        timeout
-      else if elapsed >= timeout then
-        0
-      else
-        timeout - elapsed
-    else
-      timeout
-  in
   match Erlang.term_to_binary (
     Erlang.OtpErlangTuple ([
       Erlang.OtpErlangAtom ("forward_sync");
       Erlang.OtpErlangString (name);
       Erlang.OtpErlangBinary (request_info);
       Erlang.OtpErlangBinary (request);
-      Erlang.OtpErlangInteger (timeout_new);
+      Erlang.OtpErlangInteger (timeout);
       Erlang.OtpErlangInteger (priority);
       Erlang.OtpErlangBinary (trans_id);
       Erlang.OtpErlangPid (pid)])) with
@@ -469,9 +417,9 @@ let forward_sync
     raise ForwardSync
 
 let forward_
-  api type_ name request_info request timeout priority trans_id pid :
+  api request_type name request_info request timeout priority trans_id pid :
   (unit, string) result =
-  match type_ with
+  match request_type with
   | ASYNC ->
     forward_async api name request_info request timeout priority trans_id pid
   | SYNC ->
@@ -480,21 +428,6 @@ let forward_
 let return_async_i
   api name pattern response_info response timeout trans_id pid :
   (unit, string) result =
-  let {Instance.request_timeout_adjustment;
-    request_timer; request_timeout; _} = api in
-  let timeout_new =
-    if request_timeout_adjustment && timeout = request_timeout then
-      let elapsed = truncate
-        (((Unix.gettimeofday ()) -. request_timer) *. 1000.0) in
-      if elapsed < 0 then
-        timeout
-      else if elapsed >= timeout then
-        0
-      else
-        timeout - elapsed
-    else
-      timeout
-  in
   match Erlang.term_to_binary (
     Erlang.OtpErlangTuple ([
       Erlang.OtpErlangAtom ("return_async");
@@ -502,7 +435,7 @@ let return_async_i
       Erlang.OtpErlangString (pattern);
       Erlang.OtpErlangBinary (response_info);
       Erlang.OtpErlangBinary (response);
-      Erlang.OtpErlangInteger (timeout_new);
+      Erlang.OtpErlangInteger (timeout);
       Erlang.OtpErlangBinary (trans_id);
       Erlang.OtpErlangPid (pid)])) with
   | Error (error) ->
@@ -523,21 +456,6 @@ let return_async
 let return_sync_i
   api name pattern response_info response timeout trans_id pid :
   (unit, string) result =
-  let {Instance.request_timeout_adjustment;
-    request_timer; request_timeout; _} = api in
-  let timeout_new =
-    if request_timeout_adjustment && timeout = request_timeout then
-      let elapsed = truncate
-        (((Unix.gettimeofday ()) -. request_timer) *. 1000.0) in
-      if elapsed < 0 then
-        timeout
-      else if elapsed >= timeout then
-        0
-      else
-        timeout - elapsed
-    else
-      timeout
-  in
   match Erlang.term_to_binary (
     Erlang.OtpErlangTuple ([
       Erlang.OtpErlangAtom ("return_sync");
@@ -545,7 +463,7 @@ let return_sync_i
       Erlang.OtpErlangString (pattern);
       Erlang.OtpErlangBinary (response_info);
       Erlang.OtpErlangBinary (response);
-      Erlang.OtpErlangInteger (timeout_new);
+      Erlang.OtpErlangInteger (timeout);
       Erlang.OtpErlangBinary (trans_id);
       Erlang.OtpErlangPid (pid)])) with
   | Error (error) ->
@@ -564,9 +482,9 @@ let return_sync
     raise ReturnSync
 
 let return_
-  api type_ name pattern response_info response timeout trans_id pid :
+  api request_type name pattern response_info response timeout trans_id pid :
   (unit, string) result =
-  match type_ with
+  match request_type with
   | ASYNC ->
     return_async api name pattern response_info response timeout trans_id pid
   | SYNC ->
@@ -608,14 +526,12 @@ let handle_events api ext data data_size i cmd : (bool, string) result =
               Error (error)
             | Ok (timeout_sync) ->
               let priority_default = unpack_int8 (i1 + 12) data
-              and request_timeout_adjustment = int_of_char data.[i1 + 13]
-              and i2 = i1 + 14 in
+              and i2 = i1 + 13 in
               Instance.reinit api
                 process_count
                 timeout_async
                 timeout_sync
-                priority_default
-                (request_timeout_adjustment <> 0) ;
+                priority_default ;
               loop_cmd i2)
       else if cmd_event = message_keepalive then
         match Erlang.term_to_binary (Erlang.OtpErlangAtom ("keepalive")) with
@@ -701,10 +617,7 @@ let rec poll_request_loop api timeout ext poll_timer: (bool, string) result =
 and callback
   api request_type name pattern request_info request
   timeout priority trans_id pid : (bool option, string) result =
-  let {Instance.state; callbacks; request_timeout_adjustment; _} = api in
-  if request_timeout_adjustment then (
-    api.Instance.request_timer <- Unix.gettimeofday () ;
-    api.Instance.request_timeout <- timeout) ;
+  let {Instance.state; callbacks; _} = api in
   let callback_get () =
     let function_queue = Hashtbl.find callbacks pattern in
     let f = Queue.pop function_queue in
@@ -850,9 +763,7 @@ and poll_request_data api ext data data_size i : (bool option, string) result =
                       | Ok (timeout_terminate) ->
                         let priority_default =
                           unpack_int8 (i1 + 16) data
-                        and request_timeout_adjustment =
-                          int_of_char data.[i1 + 17]
-                        and i2 = i1 + 18 in
+                        and i2 = i1 + 17 in
                         Instance.init api
                           process_index
                           process_count
@@ -863,8 +774,7 @@ and poll_request_data api ext data data_size i : (bool option, string) result =
                           timeout_async
                           timeout_sync
                           timeout_terminate
-                          priority_default
-                          (request_timeout_adjustment <> 0) ;
+                          priority_default ;
                         if i2 <> data_size then
                           match handle_events api ext data data_size i2 0 with
                           | Error (error) ->
@@ -1061,14 +971,12 @@ and poll_request_data api ext data data_size i : (bool option, string) result =
             Error (error)
           | Ok (timeout_sync) ->
             let priority_default = unpack_int8 (i + 16) data
-            and request_timeout_adjustment = int_of_char data.[i + 17]
-            and i1 = i + 18 in
+            and i1 = i + 17 in
             Instance.reinit api
               process_count
               timeout_async
               timeout_sync
-              priority_default
-              (request_timeout_adjustment <> 0) ;
+              priority_default ;
             if i1 = data_size then
               Ok (None)
             else if i1 < data_size then
@@ -1386,6 +1294,22 @@ let timeout_terminate (api : 's Instance.t) : int =
 let poll (api : 's Instance.t) (timeout : int) : (bool, string) result =
   poll_request api timeout true
 
+let shutdown
+  ?reason:(reason = "")
+  api =
+  match Erlang.term_to_binary (
+    Erlang.OtpErlangTuple ([
+      Erlang.OtpErlangAtom ("shutdown");
+      Erlang.OtpErlangString (reason)])) with
+  | Error (error) ->
+    Error (error)
+  | Ok (shutdown) ->
+    match send api shutdown with
+    | Error (error) ->
+      Error (error)
+    | Ok _ ->
+      Ok (())
+
 let text_key_value_parse text : (string, string list) Hashtbl.t =
   let result = Hashtbl.create 32
   and data = str_split_on_char '\x00' text in
