@@ -3,7 +3,7 @@
 #
 # MIT License
 #
-# Copyright (c) 2014-2018 Michael Truog <mjtruog at protonmail dot com>
+# Copyright (c) 2014-2019 Michael Truog <mjtruog at protonmail dot com>
 #
 # Permission is hereby granted, free of charge, to any person obtaining a
 # copy of this software and associated documentation files (the "Software"),
@@ -29,7 +29,7 @@ use strict;
 use warnings;
 use 5.010;
 
-$CloudI::API::VERSION = '1.75';
+$CloudI::API::VERSION = '1.80';
 
 use POSIX qw(getenv);
 use IO::Handle;
@@ -1002,6 +1002,18 @@ sub poll
     return $self->_poll_request($timeout, 1);
 }
 
+sub shutdown
+{
+    my $self = shift;
+    my ($reason) = @_;
+    if (! defined($reason))
+    {
+        $reason = '';
+    }
+    $self->_send(Erlang::term_to_binary([
+        Erlang::OtpErlangAtom->new('shutdown'), $reason]));
+}
+
 sub _text_key_value_parse
 {
     my $self = shift;
