@@ -1,41 +1,27 @@
 <?php //-*-coding:utf-8;tab-width:4;c-basic-offset:4;indent-tabs-mode:()-*-
 // ex: set ft=php fenc=utf-8 sts=4 ts=4 sw=4 et nomod:
 //
-// BSD LICENSE
+// MIT License
 //
-// Copyright (c) 2014-2017, Michael Truog <mjtruog at gmail dot com>
-// All rights reserved.
+// Copyright (c) 2014-2019 Michael Truog <mjtruog at protonmail dot com>
 //
-// Redistribution and use in source and binary forms, with or without
-// modification, are permitted provided that the following conditions are met:
+// Permission is hereby granted, free of charge, to any person obtaining a
+// copy of this software and associated documentation files (the "Software"),
+// to deal in the Software without restriction, including without limitation
+// the rights to use, copy, modify, merge, publish, distribute, sublicense,
+// and/or sell copies of the Software, and to permit persons to whom the
+// Software is furnished to do so, subject to the following conditions:
 //
-//     * Redistributions of source code must retain the above copyright
-//       notice, this list of conditions and the following disclaimer.
-//     * Redistributions in binary form must reproduce the above copyright
-//       notice, this list of conditions and the following disclaimer in
-//       the documentation and/or other materials provided with the
-//       distribution.
-//     * All advertising materials mentioning features or use of this
-//       software must display the following acknowledgment:
-//         This product includes software developed by Michael Truog
-//     * The name of the author may not be used to endorse or promote
-//       products derived from this software without specific prior
-//       written permission
+// The above copyright notice and this permission notice shall be included in
+// all copies or substantial portions of the Software.
 //
-// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
-// CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
-// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
-// OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
-// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
-// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
-// BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
-// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
-// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
-// DAMAGE.
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+// DEALINGS IN THE SOFTWARE.
 //
 
 namespace CloudI;
@@ -81,9 +67,6 @@ class API
     private $timeout_sync;
     private $timeout_terminate;
     private $priority_default;
-    private $request_timeout_adjustment;
-    private $request_timeout;
-    private $request_timer;
 
     public function __construct($thread_index)
     {
@@ -117,8 +100,7 @@ class API
              $this->timeout_async,
              $this->timeout_sync,
              $this->timeout_terminate,
-             $this->priority_default,
-             $this->request_timeout_adjustment
+             $this->priority_default
              ) = $this->poll_request(null, false);
     }
 
@@ -234,19 +216,6 @@ class API
     public function forward_async($name, $request_info, $request,
                                   $timeout, $priority, $trans_id, $pid)
     {
-        if ($this->request_timeout_adjustment)
-        {
-            if ($timeout == $this->request_timeout)
-            {
-                $elapsed = max(0, (integer) floor((microtime(true) -
-                                                   $this->request_timer) *
-                                                  1000.0));
-                if ($elapsed > $timeout)
-                    $timeout = 0;
-                else
-                    $timeout -= $elapsed;
-            }
-        }
         $this->send(\Erlang\term_to_binary(
             array(new \Erlang\OtpErlangAtom('forward_async'), $name,
                   new \Erlang\OtpErlangBinary($request_info),
@@ -258,19 +227,6 @@ class API
     public function forward_sync($name, $request_info, $request,
                                  $timeout, $priority, $trans_id, $pid)
     {
-        if ($this->request_timeout_adjustment)
-        {
-            if ($timeout == $this->request_timeout)
-            {
-                $elapsed = max(0, (integer) floor((microtime(true) -
-                                                   $this->request_timer) *
-                                                  1000.0));
-                if ($elapsed > $timeout)
-                    $timeout = 0;
-                else
-                    $timeout -= $elapsed;
-            }
-        }
         $this->send(\Erlang\term_to_binary(
             array(new \Erlang\OtpErlangAtom('forward_sync'), $name,
                   new \Erlang\OtpErlangBinary($request_info),
@@ -297,25 +253,6 @@ class API
     public function return_async($name, $pattern, $response_info, $response,
                                  $timeout, $trans_id, $pid)
     {
-        if ($this->request_timeout_adjustment)
-        {
-            if ($timeout == $this->request_timeout)
-            {
-                $elapsed = max(0, (integer) floor((microtime(true) -
-                                                   $this->request_timer) *
-                                                  1000.0));
-                if ($elapsed > $timeout)
-                {
-                    $response_info = '';
-                    $response = '';
-                    $timeout = 0;
-                }
-                else
-                {
-                    $timeout -= $elapsed;
-                }
-            }
-        }
         $this->send(\Erlang\term_to_binary(
             array(new \Erlang\OtpErlangAtom('return_async'), $name, $pattern,
                   new \Erlang\OtpErlangBinary($response_info),
@@ -327,25 +264,6 @@ class API
     public function return_sync($name, $pattern, $response_info, $response,
                                 $timeout, $trans_id, $pid)
     {
-        if ($this->request_timeout_adjustment)
-        {
-            if ($timeout == $this->request_timeout)
-            {
-                $elapsed = max(0, (integer) floor((microtime(true) -
-                                                   $this->request_timer) *
-                                                  1000.0));
-                if ($elapsed > $timeout)
-                {
-                    $response_info = '';
-                    $response = '';
-                    $timeout = 0;
-                }
-                else
-                {
-                    $timeout -= $elapsed;
-                }
-            }
-        }
         $this->send(\Erlang\term_to_binary(
             array(new \Erlang\OtpErlangAtom('return_sync'), $name, $pattern,
                   new \Erlang\OtpErlangBinary($response_info),
@@ -423,11 +341,6 @@ class API
                               $request_info, $request,
                               $timeout, $priority, $trans_id, $pid)
     {
-        if ($this->request_timeout_adjustment)
-        {
-            $this->request_timer = microtime(true);
-            $this->request_timeout = $timeout;
-        }
         if (! isset($this->callbacks[$pattern]))
         {
             $function = array($this, 'null_response');
@@ -612,18 +525,12 @@ class API
                     else
                         throw new TerminateException($this->timeout_terminate);
                 case MESSAGE_REINIT:
-                    $i += $j; $j = 4 + 4 + 4 + 1 + 1;
-                    $tmp = unpack('L3a/cb/Cc', substr($data, $i, $j));
+                    $i += $j; $j = 4 + 4 + 4 + 1;
+                    $tmp = unpack('L3a/cb', substr($data, $i, $j));
                     $this->process_count = $tmp['a1'];
                     $this->timeout_async = $tmp['a2'];
                     $this->timeout_sync = $tmp['a3'];
                     $this->priority_default = $tmp['b'];
-                    $this->request_timeout_adjustment = ($tmp['c'] == 1);
-                    if ($this->request_timeout_adjustment)
-                    {
-                        $this->request_timer = microtime(true);
-                        $this->request_timeout = 0;
-                    }
                     $i += $j;
                     break;
                 case MESSAGE_KEEPALIVE:
@@ -705,14 +612,13 @@ class API
                          $prefix_size) = unpack('L5', substr($data, $i, $j));
                     $i += $j; $j = $prefix_size;
                     $prefix = substr($data, $i, $j - 1);
-                    $i += $j; $j = 4 + 4 + 4 + 4 + 1 + 1;
-                    $tmp = unpack('L4a/cb/Cc', substr($data, $i, $j));
+                    $i += $j; $j = 4 + 4 + 4 + 4 + 1;
+                    $tmp = unpack('L4a/cb', substr($data, $i, $j));
                     $timeout_initialize = $tmp['a1'];
                     $timeout_async = $tmp['a2'];
                     $timeout_sync = $tmp['a3'];
                     $timeout_terminate = $tmp['a4'];
                     $priority_default = $tmp['b'];
-                    $request_timeout_adjustment = $tmp['c'];
                     $i += $j;
                     if ($i != $data_size)
                     {
@@ -723,8 +629,7 @@ class API
                                  $process_count_max, $process_count_min,
                                  $prefix, $timeout_initialize,
                                  $timeout_sync, $timeout_async,
-                                 $timeout_terminate, $priority_default,
-                                 ($request_timeout_adjustment == 1));
+                                 $timeout_terminate, $priority_default);
                 case MESSAGE_SEND_ASYNC:
                 case MESSAGE_SEND_SYNC:
                     $i += $j; $j = 4;
@@ -839,18 +744,12 @@ class API
                     }
                     assert(false);
                 case MESSAGE_REINIT:
-                    $i += $j; $j = 4 + 4 + 4 + 1 + 1;
-                    $tmp = unpack('L3a/cb/Cc', substr($data, $i, $j));
+                    $i += $j; $j = 4 + 4 + 4 + 1;
+                    $tmp = unpack('L3a/cb', substr($data, $i, $j));
                     $this->process_count = $tmp['a1'];
                     $this->timeout_async = $tmp['a2'];
                     $this->timeout_sync = $tmp['a3'];
                     $this->priority_default = $tmp['b'];
-                    $this->request_timeout_adjustment = ($tmp['c'] == 1);
-                    if ($this->request_timeout_adjustment)
-                    {
-                        $this->request_timer = microtime(true);
-                        $this->request_timeout = 0;
-                    }
                     $i += $j; $j = 4;
                     if ($i == $data_size)
                     {
@@ -926,6 +825,14 @@ class API
         return $this->poll_request($timeout, true);
     }
 
+    public function shutdown($reason = null)
+    {
+        if (is_null($reason))
+            $reason = "";
+        $this->send(\Erlang\term_to_binary(
+            array(new \Erlang\OtpErlangAtom('shutdown'), $reason)));
+    }
+
     private function text_key_value_parse($text)
     {
         $result = array();
