@@ -3,7 +3,7 @@
  *
  * MIT License
  *
- * Copyright (c) 2012-2017 Michael Truog <mjtruog at gmail dot com>
+ * Copyright (c) 2012-2019 Michael Truog <mjtruog at protonmail dot com>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -24,6 +24,14 @@
  * DEALINGS IN THE SOFTWARE.
  */
 
+#if defined(__clang__)
+// clang compiler specific
+#elif defined(__GNUC__)
+// GCC compiler specific
+#if __GNUC__ >= 8
+#pragma GCC diagnostic ignored "-Wcast-function-type"
+#endif
+#endif
 #include <Python.h>
 #if PY_MAJOR_VERSION >= 3
 #define PYTHON_VERSION_3_COMPATIBLE
@@ -326,6 +334,8 @@ static PyObject *
 python_cloudi_timeout_terminate(PyObject * self, PyObject *);
 static PyObject *
 python_cloudi_poll(PyObject * self, PyObject *);
+static PyObject *
+python_cloudi_shutdown(PyObject * self, PyObject * args, PyObject * kwargs);
 
 static PyMethodDef python_cloudi_instance_object_methods[] = {
     {"subscribe",
@@ -391,6 +401,9 @@ static PyMethodDef python_cloudi_instance_object_methods[] = {
     {"poll",
      python_cloudi_poll, METH_VARARGS,
      "Handle incoming requests."},
+    {"shutdown",
+     (PyCFunction) python_cloudi_shutdown, METH_VARARGS | METH_KEYWORDS,
+     "Shutdown the service successfully."},
     {NULL, NULL, 0, NULL} // Sentinel
 };
 
@@ -618,13 +631,13 @@ class callback : public CloudI::API::function_object_c
                 }
                 char const * exception_name = exception->tp_name;
                 bool const return_sync_exception =
-                    (::strcmp(exception_name, "return_sync_exception") == 0);
+                    (::strcmp(exception_name, "ReturnSyncException") == 0);
                 bool const return_async_exception =
-                    (::strcmp(exception_name, "return_async_exception") == 0);
+                    (::strcmp(exception_name, "ReturnAsyncException") == 0);
                 bool const forward_sync_exception =
-                    (::strcmp(exception_name, "forward_sync_exception") == 0);
+                    (::strcmp(exception_name, "ForwardSyncException") == 0);
                 bool const forward_async_exception =
-                    (::strcmp(exception_name, "forward_async_exception") == 0);
+                    (::strcmp(exception_name, "ForwardAsyncException") == 0);
                 bool exception_invalid = false;
                 if (request_type == CloudI::API::SYNC &&
                     return_sync_exception)
@@ -747,8 +760,9 @@ class callback : public CloudI::API::function_object_c
                 {
                     Py_ssize_t response_size_tmp = 0;
 #ifdef PYTHON_VERSION_3_3_COMPATIBLE
-                    response = PyUnicode_AsUTF8AndSize(result,
-                                                       &response_size_tmp);
+                    response =
+                        const_cast<char *>(PyUnicode_AsUTF8AndSize(
+                            result, &response_size_tmp));
                     if (response == NULL)
                     {
                         PyErr_Print();
@@ -1365,3 +1379,30 @@ python_cloudi_poll(PyObject * self, PyObject * args)
     return Py_BuildValue("O", Py_False);
 }
 
+static PyObject *
+python_cloudi_shutdown(PyObject * self, PyObject * args, PyObject * kwargs)
+{
+    python_cloudi_instance_object * object =
+        (python_cloudi_instance_object *) self;
+    char const * reason = "";
+    static char const * kwlist[] = {
+        "reason", NULL};
+    if (! PyArg_ParseTupleAndKeywords(args, kwargs,
+                                      "|s:shutdown",
+                                      const_cast<char**>(kwlist),
+                                      &reason))
+    {
+        return NULL;
+    }
+    int result;
+    THREADS_BEGIN;
+    result = object->api->shutdown(reason);
+    THREADS_END;
+    if (result != 0)
+    {
+        python_error(result);
+        return NULL;
+    }
+    Py_RETURN_NONE;
+}
+
