@@ -8,6 +8,8 @@
 
 -export([connect/1, connect/2, connect/3, connect/4,
          close/1,
+         peername/1,
+         sockname/1,
          request_info/1,
          location/1,
          request/1, request/2, request/3, request/4, request/5,
@@ -40,10 +42,10 @@
 -include("cloudi_x_hackney_internal.hrl").
 
 
--type url() :: #cloudi_x_hackney_url{} | binary().
+-type url() :: #cloudi_x_hackney_url{} | binary().
 -export_type([url/0]).
 
--opaque client() :: #client{}.
+-type client() :: #client{}.
 -export_type([client/0]).
 
 -type client_ref() :: term().
@@ -108,13 +110,22 @@ setopts(Ref, Options) ->
   cloudi_x_hackney_manager:get_state(Ref, fun(State) ->
     State2 = parse_options(Options, State),
     cloudi_x_hackney_manager:update_state(Ref, State2)
-                                 end).
+                                 end),
+  ok.
 
 %% @doc close the client
 close(Ref) ->
   cloudi_x_hackney_connect:close(Ref).
 
 
+%% @doc peername of the client
+peername(Ref) ->
+  cloudi_x_hackney_connect:peername(Ref).
+
+%% @doc sockname of the client
+sockname(Ref) ->
+  cloudi_x_hackney_connect:sockname(Ref).
+
 %% @doc get request info
 -spec request_info(client_ref()) -> list().
 request_info(Ref) ->
@@ -224,9 +235,12 @@ request(Method, URL, Headers, Body) ->
 %%          redirection for a request</li>
 %%          <li>`{force_redirect, boolean}': false by default, to force the
 %%          redirection even on POST</li>
+%%          <li>`{basic_auth, {binary, binary}}`: HTTP basic auth username and password.</li>
 %%          <li>`{proxy, proxy_options()}': to connect via a proxy.</li>
 %%          <li>`insecure': to perform "insecure" SSL connections and
 %%          transfers without checking the certificate</li>
+%%          <li>`{checkout_timeout, infinity | integer()}': timeout used when
+%%          checking out a socket from the pool, in milliseconds. Default is 8000</li>
 %%          <li>`{connect_timeout, infinity | integer()}': timeout used when
 %%          establishing a connection, in milliseconds. Default is 8000</li>
 %%          <li>`{recv_timeout, infinity | integer()}': timeout used when
@@ -955,6 +969,8 @@ reply_with_body(Status, Headers, State) ->
   case reply(Reply, State) of
     {ok, Body} ->
       {ok, Status, Headers, Body};
+    {closed, Body} ->
+      {ok, Status, Headers, Body};
     Error ->
       Error
   end.
