@@ -4,52 +4,38 @@
 %%%------------------------------------------------------------------------
 %%% @doc
 %%% ==OS Process Spawn==
-%%% Used interaction with the os_spawn process.
+%%% Used interaction with the cloudi_os_spawn process.
 %%% @end
 %%%
-%%% BSD LICENSE
-%%% 
-%%% Copyright (c) 2011-2016, Michael Truog <mjtruog at gmail dot com>
-%%% All rights reserved.
-%%% 
-%%% Redistribution and use in source and binary forms, with or without
-%%% modification, are permitted provided that the following conditions are met:
-%%% 
-%%%     * Redistributions of source code must retain the above copyright
-%%%       notice, this list of conditions and the following disclaimer.
-%%%     * Redistributions in binary form must reproduce the above copyright
-%%%       notice, this list of conditions and the following disclaimer in
-%%%       the documentation and/or other materials provided with the
-%%%       distribution.
-%%%     * All advertising materials mentioning features or use of this
-%%%       software must display the following acknowledgment:
-%%%         This product includes software developed by Michael Truog
-%%%     * The name of the author may not be used to endorse or promote
-%%%       products derived from this software without specific prior
-%%%       written permission
-%%% 
-%%% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
-%%% CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
-%%% INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
-%%% OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
-%%% DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
-%%% CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-%%% SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
-%%% BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
-%%% SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-%%% INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
-%%% WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-%%% NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-%%% OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
-%%% DAMAGE.
+%%% MIT License
 %%%
-%%% @author Michael Truog <mjtruog [at] gmail (dot) com>
-%%% @copyright 2011-2016 Michael Truog
-%%% @version 1.5.5 {@date} {@time}
+%%% Copyright (c) 2011-2019 Michael Truog <mjtruog at protonmail dot com>
+%%%
+%%% Permission is hereby granted, free of charge, to any person obtaining a
+%%% copy of this software and associated documentation files (the "Software"),
+%%% to deal in the Software without restriction, including without limitation
+%%% the rights to use, copy, modify, merge, publish, distribute, sublicense,
+%%% and/or sell copies of the Software, and to permit persons to whom the
+%%% Software is furnished to do so, subject to the following conditions:
+%%%
+%%% The above copyright notice and this permission notice shall be included in
+%%% all copies or substantial portions of the Software.
+%%%
+%%% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+%%% IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+%%% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+%%% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+%%% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+%%% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+%%% DEALINGS IN THE SOFTWARE.
+%%%
+%%% @author Michael Truog <mjtruog at protonmail dot com>
+%%% @copyright 2011-2019 Michael Truog
+%%% @version 1.8.0 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_core_i_os_spawn).
--author('mjtruog [at] gmail (dot) com').
+-author('mjtruog at protonmail dot com').
 
 -behavior(gen_server).
 
@@ -64,12 +50,12 @@
 -include("cloudi_logger.hrl").
 -include("cloudi_core_i_constants.hrl").
 -ifdef(CLOUDI_CORE_STANDALONE).
--export([spawn/14]).
+-export([spawn/15]).
 -define(ERL_PORT_NAME, "/dev/null").
 -compile({nowarn_unused_function, [{call_port_sync, 3}]}).
 spawn(_SpawnProcess, _SpawnProtocol, _SpawnSocketPath, _Ports, _SpawnRlimits,
       _SpawnUserI, _SpawnUserStr, _SpawnGroupI, _SpawnGroupStr,
-      _SpawnNice, _SpawnDirectory,
+      _SpawnNice, _SpawnChroot, _SpawnDirectory,
       _SpawnFilename, _SpawnArguments, _SpawnEnvironment) ->
     {error, badarg}.
 -else.
@@ -113,8 +99,8 @@ handle_call({call, Command, Msg}, Client,
     end;
 
 handle_call(Request, _, State) ->
-    ?LOG_ERROR("Unknown call \"~p\"~n", [Request]),
-    {stop, "Unknown call", State}.
+    {stop, cloudi_string:format("Unknown call \"~w\"", [Request]),
+     error, State}.
 
 %% handle asynchronous function calls on the port driver
 handle_cast({call, _, Msg},
@@ -130,8 +116,7 @@ handle_cast({call, _, Msg},
     {noreply, State};
 
 handle_cast(Request, State) ->
-    ?LOG_WARN("Unknown cast \"~p\"~n", [Request]),
-    {noreply, State}.
+    {stop, cloudi_string:format("Unknown cast \"~w\"", [Request]), State}.
 
 %% port exited with a fatal error/signal
 handle_info({Port, {exit_status, Status}},
@@ -270,8 +255,7 @@ handle_info({ReplyRef, _}, State) when is_reference(ReplyRef) ->
     {noreply, State};
 
 handle_info(Request, State) ->
-    ?LOG_WARN("Unknown info \"~p\"~n", [Request]),
-    {noreply, State}.
+    {stop, cloudi_string:format("Unknown info \"~w\"", [Request]), State}.
 
 terminate(_, #state{port = Port}) when is_port(Port) ->
     catch erlang:port_close(Port),
@@ -290,31 +274,21 @@ code_change(_, State, _) ->
 -ifdef(ERL_PORT_DRIVER_NAME).
 local_port_name() ->
     ?ERL_PORT_DRIVER_NAME.
-%local_port_name_prefix() ->
-%    ?ERL_PORT_DRIVER_NAME_PREFIX.
-%unload_local_port(Name) when is_list(Name) ->
-%    % assume only one process owns the driver
-%    % (otherwise more complex logic is necessary)
-%    case erl_ddll:try_unload(Name, []) of
-%        {ok, unloaded} ->
-%            ok;
-%        {ok, Error} when is_atom(Error) ->
-%            {error, atom_to_list(Error)};
-%        {error, Error} when is_atom(Error) ->
-%            {error, atom_to_list(Error)}
-%    end.
 load_local_port(Name) when is_list(Name) ->
     {ok, Path} = load_path(Name ++ ".so"),
     Result = case erl_ddll:load_driver(Path, Name) of
-        ok -> ok;
-        {error, already_loaded} -> ok;
+        ok ->
+            ok;
+        {error, already_loaded} ->
+            ok;
         % {open_error, -10}, otherwise known as "Unknown dlload error",
         % could mean any of the following:
         % - driver does not exist in the path specified
         % - driver can not be read
         % - driver does not have all the required symbols
         % etc.
-        {error, ErrorDesc} -> {error, erl_ddll:format_error(ErrorDesc)}
+        {error, ErrorDesc} ->
+            {error, erl_ddll:format_error(ErrorDesc)}
     end,
     case Result of
         ok ->
@@ -329,8 +303,7 @@ load_local_port(Name) when is_list(Name) ->
     end.
 transform_data(D) ->
     D.
-%% only a port driver can perform asynchronous function calls
-%% (you might be able to put a thread pool in an Erlang port, but why bother?)
+% only a port driver can perform asynchronous function calls
 call_port_async(Process, Command, Msg)
     when is_integer(Command), is_list(Msg) ->
     gen_server:cast(Process, {call, Command, Msg}).
@@ -338,10 +311,6 @@ call_port_async(Process, Command, Msg)
 -ifdef(ERL_PORT_NAME).
 local_port_name() ->
     ?ERL_PORT_NAME.
-%local_port_name_prefix() ->
-%    ?ERL_PORT_NAME_PREFIX.
-%unload_local_port(Name) when is_list(Name) ->
-%    ok.
 load_local_port(Name) when is_list(Name) ->
     {ok, Path} = load_path(Name),
     erlang:open_port({spawn, Path ++ "/" ++ Name},
@@ -363,15 +332,19 @@ call_port_sync(Process, Command, Msg)
 
 call_port(Port, Msg) when is_port(Port), is_list(Msg) ->
     try erlang:port_command(Port, Msg) of
-        true -> ok
+        true ->
+            ok
     catch
         error:badarg ->
             try erlang:iolist_size(Msg) of
-                _ -> {error, einval}
+                _ ->
+                    {error, einval}
             catch
-                error:_ -> {error, badarg}
+                error:_ ->
+                    {error, badarg}
             end;
-        error:Reason -> {error, Reason}
+        error:Reason ->
+            {error, Reason}
     end.
 
 load_path(File) when is_list(File) ->
