@@ -8,7 +8,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2014-2017 Michael Truog <mjtruog at protonmail dot com>
+%%% Copyright (c) 2014-2018 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -29,8 +29,8 @@
 %%% DEALINGS IN THE SOFTWARE.
 %%%
 %%% @author Michael Truog <mjtruog at protonmail dot com>
-%%% @copyright 2014-2017 Michael Truog
-%%% @version 1.7.1 {@date} {@time}
+%%% @copyright 2014-2018 Michael Truog
+%%% @version 1.7.5 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_service_name).
@@ -41,6 +41,7 @@
          new/4,
          parse/2,
          parse_with_suffix/2,
+         pattern/1,
          suffix/2]).
 
 %%%------------------------------------------------------------------------
@@ -61,7 +62,7 @@
     {error, parameters_ignored | parameter_missing}.
 
 new(Pattern, Parameters) ->
-    new_insert(Pattern, Parameters, true).
+    cloudi_x_trie:pattern2_fill(Pattern, Parameters).
 
 %%-------------------------------------------------------------------------
 %% @doc
@@ -81,11 +82,9 @@ new(Pattern, Parameters) ->
      {parameters_selected_ignored, list(pos_integer())} |
      {parameters_selected_missing, pos_integer()}}.
 
-new(Pattern, Parameters, [], ParametersStrictMatching) ->
-    new_insert(Pattern, Parameters, ParametersStrictMatching);
 new(Pattern, Parameters, ParametersSelected, ParametersStrictMatching) ->
-    new_select(Pattern, Parameters, ParametersSelected,
-               ParametersStrictMatching).
+    cloudi_x_trie:pattern2_fill(Pattern, Parameters,
+                                ParametersSelected, ParametersStrictMatching).
 
 %%-------------------------------------------------------------------------
 %% @doc
@@ -98,7 +97,7 @@ new(Pattern, Parameters, ParametersSelected, ParametersStrictMatching) ->
     list(string()) | error.
 
 parse(Name, Pattern) ->
-    cloudi_x_trie:pattern_parse(Pattern, Name).
+    cloudi_x_trie:pattern2_parse(Pattern, Name).
 
 %%-------------------------------------------------------------------------
 %% @doc
@@ -111,7 +110,21 @@ parse(Name, Pattern) ->
     {list(string()), string()} | error.
 
 parse_with_suffix(Name, Pattern) ->
-    cloudi_x_trie:pattern_parse(Pattern, Name, with_suffix).
+    cloudi_x_trie:pattern2_parse(Pattern, Name, with_suffix).
+
+%%-------------------------------------------------------------------------
+%% @doc
+%% ===Determine if a service name pattern contains wildcard characters.===
+%% If not, the service name pattern would only be used with a service name
+%% that is an exact match.
+%% @end
+%%-------------------------------------------------------------------------
+
+-spec pattern(Pattern :: string()) ->
+    boolean().
+
+pattern(Pattern) ->
+    cloudi_x_trie:is_pattern2(Pattern).
 
 %%-------------------------------------------------------------------------
 %% @doc
@@ -125,13 +138,13 @@ parse_with_suffix(Name, Pattern) ->
 
 suffix([PrefixC | _] = Prefix, [NameOrPatternC | _] = NameOrPattern)
     when is_integer(PrefixC), is_integer(NameOrPatternC) ->
-    case lists:member($*, NameOrPattern) of
+    case cloudi_x_trie:is_pattern2(NameOrPattern) of
         true ->
             % handle as a pattern
             suffix_pattern_parse(Prefix, NameOrPattern);
         false ->
             % handle as a name
-            case cloudi_x_trie:pattern_suffix(Prefix, NameOrPattern) of
+            case cloudi_x_trie:pattern2_suffix(Prefix, NameOrPattern) of
                 error ->
                     erlang:exit(badarg);
                 Suffix ->
@@ -145,87 +158,6 @@ suffix(_, _) ->
 %%% Private functions
 %%%------------------------------------------------------------------------
 
-new_strip([], NameOut) ->
-    {ok, lists:reverse(NameOut)};
-new_strip([$* | PatternIn], NameOut) ->
-    new_strip(PatternIn, NameOut);
-new_strip([C | PatternIn], NameOut) ->
-    new_strip(PatternIn, [C | NameOut]).
-
-new_insert([], NameOut,
-           [], _) ->
-    {ok, lists:reverse(NameOut)};
-new_insert([], NameOut,
-           [_ | _], ParametersStrictMatching) ->
-    if
-        ParametersStrictMatching =:= true ->
-            {error, parameters_ignored};
-        true ->
-            {ok, lists:reverse(NameOut)}
-    end;
-new_insert([$* | PatternIn], NameOut,
-           [], ParametersStrictMatching) ->
-    if
-        ParametersStrictMatching =:= true ->
-            {error, parameter_missing};
-        true ->
-            new_strip(PatternIn, NameOut)
-    end;
-new_insert([$* | PatternIn], NameOut,
-           [Parameter | Parameters], ParametersStrictMatching) ->
-    new_insert(PatternIn, lists:reverse(Parameter) ++ NameOut,
-               Parameters, ParametersStrictMatching);
-new_insert([C | PatternIn], NameOut,
-           Parameters, ParametersStrictMatching) ->
-    new_insert(PatternIn, [C | NameOut],
-               Parameters, ParametersStrictMatching).
-
-new_insert(PatternIn, Parameters, ParametersStrictMatching) ->
-    new_insert(PatternIn, [], Parameters, ParametersStrictMatching).
-
-new_select([], NameOut, _,
-           ParametersSelected, ParametersStrictMatching) ->
-    if
-        ParametersStrictMatching =:= true, ParametersSelected /= [] ->
-            {error, {parameters_selected_ignored, ParametersSelected}};
-        true ->
-            {ok, lists:reverse(NameOut)}
-    end;
-new_select([$* | PatternIn], NameOut, _,
-           [], ParametersStrictMatching) ->
-    if
-        ParametersStrictMatching =:= true ->
-            {error, parameters_selected_empty};
-        true ->
-            new_strip(PatternIn, NameOut)
-    end;
-new_select([$* | PatternIn], NameOut, Parameters,
-           [I | ParametersSelected],
-           ParametersStrictMatching) ->
-    try lists:nth(I, Parameters) of
-        Parameter ->
-            new_select(PatternIn, lists:reverse(Parameter) ++ NameOut,
-                       Parameters, ParametersSelected,
-                       ParametersStrictMatching)
-    catch
-        error:_ ->
-            if
-                ParametersStrictMatching =:= true ->
-                    {error, {parameters_selected_missing, I}};
-                true ->
-                    new_strip(PatternIn, NameOut)
-            end
-    end;
-new_select([C | PatternIn], NameOut, Parameters,
-           ParametersSelected, ParametersStrictMatching) ->
-    new_select(PatternIn, [C | NameOut], Parameters,
-               ParametersSelected, ParametersStrictMatching).
-
-new_select(PatternIn, Parameters,
-           ParametersSelected, ParametersStrictMatching) ->
-    new_select(PatternIn, [], Parameters,
-               ParametersSelected, ParametersStrictMatching).
-
 suffix_pattern_parse([], Pattern) ->
     Pattern;
 suffix_pattern_parse([C | Prefix], [C | Pattern]) ->
