@@ -7,7 +7,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2014-2018 Michael Truog <mjtruog at protonmail dot com>
+%%% Copyright (c) 2014-2019 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -28,8 +28,8 @@
 %%% DEALINGS IN THE SOFTWARE.
 %%%
 %%% @author Michael Truog <mjtruog at protonmail dot com>
-%%% @copyright 2014-2018 Michael Truog
-%%% @version 1.7.5 {@date} {@time}
+%%% @copyright 2014-2019 Michael Truog
+%%% @version 1.8.0 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_service_SUITE).
@@ -68,6 +68,7 @@
          t_service_internal_terminate_4/1,
          t_service_internal_update_1/1,
          t_service_internal_log_1/1,
+         t_service_internal_idle_1/1,
          t_cloudi_args_type_1/1,
          t_cloudi_service_name_1/1]).
 
@@ -155,6 +156,8 @@ cloudi_service_init(Args, ?SERVICE_PREFIX1, _Timeout, Dispatcher) ->
                                                       undefined, undefined),
             {ok, _, _} = cloudi_service:recv_async(Dispatcher, TransId),
             undefined;
+        Mode =:= idle ->
+            idle;
         Mode =:= terminate_sleep ->
             cloudi_service:subscribe(Dispatcher,
                                      ?SERVICE_SUFFIX1),
@@ -162,23 +165,23 @@ cloudi_service_init(Args, ?SERVICE_PREFIX1, _Timeout, Dispatcher) ->
     end,
     {ok, #state{mode = NewMode}}.
 
-cloudi_service_handle_request(Type, Name, Pattern,
+cloudi_service_handle_request(RequestType, Name, Pattern,
                               ?REQUEST_INFO1 = RequestInfo, ?REQUEST1 = Request,
                               Timeout, Priority, TransId, Pid,
                               #state{mode = reply,
                                      requests = Requests} = State,
                               _Dispatcher) ->
-    NewRequests = [{Type, Name, Pattern, RequestInfo, Request,
+    NewRequests = [{RequestType, Name, Pattern, RequestInfo, Request,
                     Timeout, Priority, TransId, Pid} | Requests],
     {reply, ?RESPONSE_INFO1, ?RESPONSE1,
      State#state{requests = NewRequests}};
-cloudi_service_handle_request(_Type, _Name, _Pattern,
+cloudi_service_handle_request(_RequestType, _Name, _Pattern,
                               ?REQUEST_INFO2, ?REQUEST2,
                               _Timeout, _Priority, _TransId, _Pid,
                               #state{requests = Requests} = State,
                               _Dispatcher) ->
     {reply, <<>>, lists:reverse(Requests), State#state{requests = []}};
-cloudi_service_handle_request(_Type, Name, _Pattern,
+cloudi_service_handle_request(_RequestType, Name, _Pattern,
                               ?REQUEST_INFO3, ?REQUEST3,
                               _Timeout, _Priority, _TransId, _Pid,
                               #state{mode = reply} = State,
@@ -188,13 +191,13 @@ cloudi_service_handle_request(_Type, Name, _Pattern,
                                                 undefined, undefined),
     {ok, Responses} = cloudi_service:recv_asyncs(Dispatcher, TransIds),
     {reply, Responses, State};
-cloudi_service_handle_request(_Type, _Name, _Pattern,
+cloudi_service_handle_request(_RequestType, _Name, _Pattern,
                               ?REQUEST_INFO4, ?REQUEST4,
                               _Timeout, _Priority, _TransId, _Pid,
                               #state{} = State,
                               _Dispatcher) ->
     {reply, ?RESPONSE_INFO1, ?RESPONSE1, State};
-cloudi_service_handle_request(_Type, _Name, _Pattern,
+cloudi_service_handle_request(_RequestType, _Name, _Pattern,
                               ?REQUEST_INFO5, ?REQUEST5,
                               _Timeout, _Priority, _TransId, _Pid,
                               #state{mode = reply,
@@ -202,13 +205,13 @@ cloudi_service_handle_request(_Type, _Name, _Pattern,
                               _Dispatcher) ->
     {reply, ?RESPONSE_INFO1, ?RESPONSE1,
      State#state{count = Count + 1}};
-cloudi_service_handle_request(_Type, _Name, _Pattern,
+cloudi_service_handle_request(_RequestType, _Name, _Pattern,
                               ?REQUEST_INFO6, ?REQUEST6,
                               _Timeout, _Priority, _TransId, _Pid,
                               #state{count = Count} = State,
                               _Dispatcher) ->
     {reply, Count, State};
-cloudi_service_handle_request(_Type, _Name, _Pattern,
+cloudi_service_handle_request(_RequestType, _Name, _Pattern,
                               ?REQUEST_INFO7, ?REQUEST7,
                               _Timeout, _Priority, _TransId, _Pid,
                               #state{} = State,
@@ -278,7 +281,8 @@ groups() ->
        t_service_internal_terminate_3,
        t_service_internal_terminate_4,
        t_service_internal_update_1,
-       t_service_internal_log_1]},
+       t_service_internal_log_1,
+       t_service_internal_idle_1]},
      {cloudi_modules_1, [parallel],
       [t_cloudi_args_type_1,
        t_cloudi_service_name_1]}].
@@ -482,6 +486,18 @@ init_per_testcase(TestCase, Config)
            {duo_mode, true}]}]
         ], infinity),
     [{service_ids, ServiceIds} | Config];
+init_per_testcase(TestCase, Config)
+    when (TestCase =:= t_service_internal_idle_1) ->
+    init_per_testcase(TestCase),
+    {ok, ServiceIds} = cloudi_service_api:services_add([
+        % using proplist configuration format, not the tuple/record format
+        [{prefix, ?SERVICE_PREFIX1},
+         {module, ?MODULE},
+         {args, [{mode, idle}]},
+         {options,
+          [{automatic_loading, false}]}]
+        ], infinity),
+    [{service_ids, ServiceIds} | Config];
 init_per_testcase(TestCase, Config) ->
     init_per_testcase(TestCase),
     Config.
@@ -920,6 +936,14 @@ t_service_internal_log_1(_Config) ->
     ?LOG_INFO("Logging metadata", []),
     ok.
 
+t_service_internal_idle_1(Config) ->
+    [ServiceId] = ?config(service_ids, Config),
+    {ok, []} = cloudi_service_api:
+               service_subscriptions(ServiceId, infinity),
+    {error, not_found} = cloudi_service_api:
+                         service_subscriptions(<<0:128>>, infinity),
+    ok.
+
 t_cloudi_args_type_1(_Config) ->
     % based on cloudi_service_name:suffix/2 but enforcing checks on whether
     % it is a service name or service name pattern for service initialization
