@@ -8,7 +8,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2009-2017 Michael Truog <mjtruog at gmail dot com>
+%%% Copyright (c) 2009-2019 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -28,13 +28,13 @@
 %%% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 %%% DEALINGS IN THE SOFTWARE.
 %%%
-%%% @author Michael Truog <mjtruog [at] gmail (dot) com>
-%%% @copyright 2009-2017 Michael Truog
-%%% @version 1.7.1 {@date} {@time}
+%%% @author Michael Truog <mjtruog at protonmail dot com>
+%%% @copyright 2009-2019 Michael Truog
+%%% @version 1.8.0 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_service_db_mysql).
--author('mjtruog [at] gmail (dot) com').
+-author('mjtruog at protonmail dot com').
 
 -behaviour(cloudi_service).
 
@@ -73,6 +73,21 @@
 % supported drivers
 -define(MODULE_EONBLAST, cloudi_x_emysql_conn). % default
 
+-define(DRIVER_DEBUG(Level, Query, Parameters, Result),
+    ?LOG(Level,
+         "SQL(equery):~n"
+         " ~p~n"
+         " ~p~n"
+         " = ~p",
+         [Query, Parameters, Result])).
+
+-define(DRIVER_DEBUG(Level, Query, Result),
+    ?LOG(Level,
+         "SQL(squery):~n"
+         " ~p~n"
+         " = ~p",
+         [Query, Result])).
+
 -record(state,
     {
         module :: ?MODULE_EONBLAST,
@@ -150,6 +165,29 @@
         extra :: binary()
     }).
 
+% for features specific to Erlang/OTP version 21.x (and later versions)
+-ifdef(ERLANG_OTP_VERSION_19).
+-else.
+-ifdef(ERLANG_OTP_VERSION_20).
+-else.
+-ifdef(OTP_RELEASE).
+-define(ERLANG_OTP_VERSION_21_FEATURES, true).
+-else.
+-error("Erlang/OTP version invalid").
+-endif.
+-endif.
+-endif.
+
+% Get the stacktrace in a way that is backwards compatible
+-ifdef(ERLANG_OTP_VERSION_21_FEATURES).
+-define(STACKTRACE(ErrorType, Error, ErrorStackTrace),
+        ErrorType:Error:ErrorStackTrace ->).
+-else.
+-define(STACKTRACE(ErrorType, Error, ErrorStackTrace),
+        ErrorType:Error ->
+            ErrorStackTrace = erlang:get_stacktrace(),).
+-endif.
+
 %%%------------------------------------------------------------------------
 %%% External interface functions
 %%%------------------------------------------------------------------------
@@ -427,7 +465,8 @@ cloudi_service_init(Args, _Prefix, Timeout, Dispatcher) ->
             {stop, Reason}
     end.
 
-cloudi_service_handle_request(_Type, _Name, _Pattern, _RequestInfo, Request,
+cloudi_service_handle_request(_RequestType, _Name, _Pattern,
+                              _RequestInfo, Request,
                               Timeout, _Priority, _TransId, _Pid,
                               #state{output_type = OutputType,
                                      external_format = ExternalFormat} = State,
@@ -482,11 +521,9 @@ cloudi_service_handle_info({ping, Ping} = Request, State, Dispatcher) ->
             {stop, ping_failed, State}
     end;
 cloudi_service_handle_info(Request, State, _Dispatcher) ->
-    ?LOG_WARN("Unknown info \"~p\"", [Request]),
-    {noreply, State}.
+    {stop, cloudi_string:format("Unknown info \"~w\"", [Request]), State}.
 
-cloudi_service_terminate(_Reason, _Timeout,
-                         undefined) ->
+cloudi_service_terminate(_Reason, _Timeout, undefined) ->
     ok;
 cloudi_service_terminate(_Reason, _Timeout,
                          #state{module = Module,
@@ -587,9 +624,9 @@ driver_open(?MODULE_EONBLAST, HostName, UserName, Password,
             % (not using the ?MODULE_EONBLAST connection manager process)
             {ok, Connection}
     catch
-        ErrorType:Error ->
+        ?STACKTRACE(ErrorType, Error, ErrorStackTrace)
             ?LOG_ERROR("start failed ~p ~p~n~p",
-                       [ErrorType, Error, erlang:get_stacktrace()]),
+                       [ErrorType, Error, ErrorStackTrace]),
             {error, Error}
     end.
 
@@ -636,12 +673,7 @@ driver_equery(Query, Parameters, Timeout, ResponseOutputType,
     end,
     Native = ?MODULE_EONBLAST:execute(Connection,
                                       NewQuery, Parameters, Timeout),
-    if
-        DebugLevel =:= off ->
-            ok;
-        true ->
-            driver_debug(DebugLevel, Query, Parameters, Native)
-    end,
+    ?DRIVER_DEBUG(DebugLevel, Query, Parameters, Native),
     Response = if
         Interface =:= common; ResponseOutputType =:= external ->
             eonblast_to_common(Native);
@@ -669,12 +701,7 @@ driver_squery(Query, Timeout, ResponseOutputType,
     end,
     Native = ?MODULE_EONBLAST:execute(Connection,
                                       QueryBinary, [], Timeout),
-    if
-        DebugLevel =:= off ->
-            ok;
-        true ->
-            driver_debug(DebugLevel, Query, Native)
-    end,
+    ?DRIVER_DEBUG(DebugLevel, Query, Native),
     Response = if
         Interface =:= common; ResponseOutputType =:= external ->
             eonblast_to_common(Native);
@@ -707,17 +734,12 @@ driver_prepare(Identifier, Query, Timeout, ResponseOutputType,
         _:Reason ->
             {error, Reason}
     end,
-    if
-        DebugLevel =:= off ->
-            ok;
-        true ->
-            driver_debug_log(DebugLevel,
-                             "prepare:~n"
-                             " ~p~n"
-                             " ~p~n"
-                             " = ~p",
-                             [Identifier, Query, Response])
-    end,
+    ?LOG(DebugLevel,
+         "prepare:~n"
+         " ~p~n"
+         " ~p~n"
+         " = ~p",
+         [Identifier, Query, Response]),
     if
         ResponseOutputType =:= internal ->
             Response;
@@ -735,45 +757,12 @@ driver_ping(Timeout,
                    connection = Connection,
                    debug_level = DebugLevel}) ->
     Result = ?MODULE_EONBLAST:ping_connection(Connection, Timeout),
-    if
-        DebugLevel =:= off ->
-            ok;
-        true ->
-            driver_debug_log(DebugLevel,
-                             "ping:~n"
-                             " = ~p",
-                             [Result])
-    end,
+    ?LOG(DebugLevel,
+         "ping:~n"
+         " = ~p",
+         [Result]),
     Result.
 
-driver_debug_log(trace, Message, Args) ->
-    ?LOG_TRACE(Message, Args);
-driver_debug_log(debug, Message, Args) ->
-    ?LOG_DEBUG(Message, Args);
-driver_debug_log(info, Message, Args) ->
-    ?LOG_INFO(Message, Args);
-driver_debug_log(warn, Message, Args) ->
-    ?LOG_WARN(Message, Args);
-driver_debug_log(error, Message, Args) ->
-    ?LOG_ERROR(Message, Args);
-driver_debug_log(fatal, Message, Args) ->
-    ?LOG_FATAL(Message, Args).
-
-driver_debug(Level, Query, Parameters, Result) ->
-    driver_debug_log(Level,
-                     "SQL(equery):~n"
-                     " ~p~n"
-                     " ~p~n"
-                     " = ~p",
-                     [Query, Parameters, Result]).
-
-driver_debug(Level, Query, Result) ->
-    driver_debug_log(Level,
-                     "SQL(squery):~n"
-                     " ~p~n"
-                     " = ~p",
-                     [Query, Result]).
-
 eonblast_to_common(L) when is_list(L) ->
     [H | _] = lists:reverse(L),
     eonblast_to_common(H);
