@@ -10,7 +10,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2014-2018 Michael Truog <mjtruog at protonmail dot com>
+%%% Copyright (c) 2014-2019 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -31,8 +31,8 @@
 %%% DEALINGS IN THE SOFTWARE.
 %%%
 %%% @author Michael Truog <mjtruog at protonmail dot com>
-%%% @copyright 2014-2018 Michael Truog
-%%% @version 1.7.4 {@date} {@time}
+%%% @copyright 2014-2019 Michael Truog
+%%% @version 1.8.0 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_service_oauth1).
@@ -113,7 +113,11 @@
 -else.
 -ifdef(ERLANG_OTP_VERSION_20).
 -else.
+-ifdef(OTP_RELEASE).
 -define(ERLANG_OTP_VERSION_21_FEATURES, true).
+-else.
+-error("Erlang/OTP version invalid").
+-endif.
 -endif.
 -endif.
 
@@ -217,7 +221,7 @@ cloudi_service_init(Args, Prefix, _Timeout, Dispatcher) ->
             true = is_boolean(DebugDB0),
             DebugDB0
     end,
-    false = lists:member($*, Prefix),
+    false = cloudi_service_name:pattern(Prefix),
     % endpoints based on http://tools.ietf.org/html/rfc5849#section-1.2
     ok = cloudi_service:subscribe(Dispatcher, "initiate/post"),
     ok = cloudi_service:subscribe(Dispatcher, "authorize/get"),
@@ -247,7 +251,7 @@ cloudi_service_init(Args, Prefix, _Timeout, Dispatcher) ->
                 debug = Debug,
                 prefix_length = erlang:length(Prefix)}}.
 
-cloudi_service_handle_request(_Type, Name, Pattern, RequestInfo, Request,
+cloudi_service_handle_request(_RequestType, Name, Pattern, RequestInfo, Request,
                               Timeout, _Priority, _TransId, _Pid,
                               #state{prefix_length = PrefixLength} = State,
                               Dispatcher) ->
@@ -381,7 +385,7 @@ url(Method, RequestHeaders, URLHost, Host, Request) ->
     case cloudi_key_value:find(<<"host">>, RequestHeaders) of
         {ok, Host} when Method == "GET" ->
             {ok, URLHost ++ erlang:binary_to_list(URLPath) ++ "?" ++
-                 erlang:binary_to_list(cloudi_x_cow_qs:qs(Request))};
+                 erlang:binary_to_list(cloudi_x_cow1_qs:qs(Request))};
         {ok, Host} ->
             {ok, URLHost ++ erlang:binary_to_list(URLPath)};
         {ok, HostInvalid} ->
@@ -708,7 +712,7 @@ callback_merge_qs_filter_request([{K, _} = Entry |
 callback_merge(CallbackURL, CallbackQS, RequestQS, Verifier) ->
     NewCallbackQS = callback_merge_qs_filter_callback(CallbackQS),
     NewRequestQS = callback_merge_qs_filter_request(RequestQS, CallbackQS),
-    QS = cloudi_x_cow_qs:qs(NewCallbackQS ++ NewRequestQS ++
+    QS = cloudi_x_cow1_qs:qs(NewCallbackQS ++ NewRequestQS ++
                    [{<<"oauth_verifier">>, Verifier}]),
     <<CallbackURL/binary, $?, QS/binary>>.
 
@@ -746,7 +750,7 @@ request_authorize(RequestQS, TokenRequest, Timeout,
             end;
         {ok, CallbackURL, CallbackQS} ->
             Callback = callback_merge(CallbackURL,
-                                      cloudi_x_cow_qs:parse_qs(CallbackQS),
+                                      cloudi_x_cow1_qs:parse_qs(CallbackQS),
                                       RequestQS, Verifier),
             case DatabaseModule:token_request_update(Dispatcher, Database,
                                                      TokenRequest, Verifier,
