@@ -7,7 +7,9 @@
 
 REBAR_DIRS="\
     lib/cloudi_core \
+    lib/cloudi_service_api_batch \
     lib/cloudi_service_api_requests \
+    lib/cloudi_service_cron \
     lib/cloudi_service_db_mysql \
     lib/cloudi_service_db_pgsql \
     lib/cloudi_service_filesystem \
@@ -23,6 +25,7 @@ REBAR_DIRS="\
     lib/cloudi_service_quorum \
     lib/cloudi_service_request_rate \
     lib/cloudi_service_router \
+    lib/cloudi_service_shell \
     lib/cloudi_service_tcp \
     lib/cloudi_service_udp \
     lib/cloudi_service_validate \
